<?php

/**
 * @param $fallback
 * @return array
 * @throws ErrorException
 */
function route($fallback)
{
    $page = filter_input(INPUT_GET, 'p', FILTER_SANITIZE_SPECIAL_CHARS);
    $categorie = filter_input(INPUT_GET, 'c', FILTER_SANITIZE_SPECIAL_CHARS);
    $option = filter_input(INPUT_GET, 'o', FILTER_SANITIZE_SPECIAL_CHARS);

    switch ($page) {
        case 'login':
            switch ($option) {
                case 'steam':
                    require_once 'includes/auth/openid.php';
                    new SteamAuthHandler(new LightOpenID(Host::complete()), $GLOBALS['PDO']);
                    exit();
                default:
                    return ['Connexion', '/page.login.php'];
            }
        case 'logout':
            Auth::logout();
            header('Location: index.php?p=home');
            exit();
        case 'submit':
            return ['Soumettre un ban', '/page.submit.php'];
        case 'banlist':
            return ['Liste des bans', '/page.banlist.php'];
        case 'commslist':
            return ['Liste des mutes', '/page.commslist.php'];
        case 'servers':
            return ['Liste des serveurs', '/page.servers.php'];
        case 'protest':
            return ['Protester un ban', '/page.protest.php'];
        case 'account':
            return ['Votre compte', '/page.youraccount.php'];
        case 'lostpassword':
            return ['Mot de passe perdu', '/page.lostpassword.php'];
        case 'home':
            return ['Tableau de bord', '/page.home.php'];
        case 'admin':
            switch ($categorie) {
                case 'groups':
                    CheckAdminAccess(ADMIN_OWNER|ADMIN_LIST_GROUPS|ADMIN_ADD_GROUP|ADMIN_EDIT_GROUPS|ADMIN_DELETE_GROUPS);
                    switch ($option) {
                        case 'edit':
                            return ['Éditer les groupes', '/admin.edit.group.php'];
                        default:
                            return ['Gestion des groupes', '/admin.groups.php'];
                    }
                case 'admins':
                    CheckAdminAccess(ADMIN_OWNER|ADMIN_LIST_ADMINS|ADMIN_ADD_ADMINS|ADMIN_EDIT_ADMINS|ADMIN_DELETE_ADMINS);
                    switch ($option) {
                        case 'editgroup':
                            return ['Éditer les groupes de l\'admin', '/admin.edit.admingroup.php'];
                        case 'editdetails':
                            return ['Éditer les détails de l\'admin', '/admin.edit.admindetails.php'];
                        case 'editpermissions':
                            return ['Éditer les permissions', '/admin.edit.adminperms.php'];
                        case 'editservers':
                            return ['Éditer les accès serveur', '/admin.edit.adminservers.php'];
                        default:
                            return ['Gestion des admins', '/admin.admins.php'];
                    }
                case 'servers':
                    CheckAdminAccess(ADMIN_OWNER|ADMIN_LIST_SERVERS|ADMIN_ADD_SERVER|ADMIN_EDIT_SERVERS|ADMIN_DELETE_SERVERS);
                    switch ($option) {
                        case 'edit':
                            return ['Éditer le serveur', '/admin.edit.server.php'];
                        case 'rcon':
                            return ['RCON serveur', '/admin.rcon.php'];
                        case 'admincheck':
                            return ['Admins serveur', '/admin.srvadmins.php'];
                        default:
                            return ['Gestion des serveurs', '/admin.servers.php'];
                    }
                case 'bans':
                    CheckAdminAccess(ADMIN_OWNER|ADMIN_ADD_BAN|ADMIN_EDIT_OWN_BANS|ADMIN_EDIT_GROUP_BANS|ADMIN_EDIT_ALL_BANS|ADMIN_BAN_PROTESTS|ADMIN_BAN_SUBMISSIONS);
                    switch ($option) {
                        case 'edit':
                            return ['Éditer les détails du ban', '/admin.edit.ban.php'];
                        case 'email':
                            return ['Email', '/admin.email.php'];
                        default:
                            return ['Bans', '/admin.bans.php'];
                    }
                case 'comms':
                    CheckAdminAccess(ADMIN_OWNER|ADMIN_ADD_BAN|ADMIN_EDIT_OWN_BANS|ADMIN_EDIT_ALL_BANS);
                    switch ($option) {
                        case 'edit':
                            return ['Éditer les détails du mute', '/admin.edit.comms.php'];
                        default:
                            return ['Mutes', '/admin.comms.php'];
                    }
                case 'mods':
                    CheckAdminAccess(ADMIN_OWNER|ADMIN_LIST_MODS|ADMIN_ADD_MODS|ADMIN_EDIT_MODS|ADMIN_DELETE_MODS);
                    switch ($option) {
                        case 'edit':
                            return ['Éditer les détails du mod', '/admin.edit.mod.php'];
                        default:
                            return ['Gestion des mods', '/admin.mods.php'];
                    }
                case 'settings':
                    CheckAdminAccess(ADMIN_OWNER|ADMIN_WEB_SETTINGS);
                    return ['Paramètres SourceBans++', '/admin.settings.php'];
                default:
                    CheckAdminAccess(ALL_WEB);
                    return ['Administration', '/page.admin.php'];
        }
        default:
            switch ($fallback) {
                case 1:
                    $_GET['p'] = 'banlist';
                    return ['Liste des bans', '/page.banlist.php'];
                case 2:
                    $_GET['p'] = 'servers';
                    return ['Info serveur', '/page.servers.php'];
                case 3:
                    $_GET['p'] = 'submit';
                    return ['Soumettre un ban', '/page.submit.php'];
                case 4:
                    $_GET['p'] = 'protest';
                    return ['Protester un ban', '/page.protest.php'];
                default:
                    $_GET['p'] = 'home';
                    return ['Tableau de bord', '/page.home.php'];
            }
    }
}

/**
 * @param null $title Unused
 * @param string $page
 */
function build(string $title, string $page)
{
    require_once(TEMPLATES_PATH.'/core/header.php');
    require_once(TEMPLATES_PATH.'/core/navbar.php');
    require_once(TEMPLATES_PATH.'/core/title.php');
    require_once(TEMPLATES_PATH.$page);
    require_once(TEMPLATES_PATH.'/core/footer.php');
}
