<?php
/*************************************************************************
This file is part of SourceBans++

SourceBans++ (c) 2014-2023 by SourceBans++ Dev Team

The SourceBans++ Web panel is licensed under a
Creative Commons Attribution-NonCommercial-ShareAlike 3.0 Unported License.

You should have received a copy of the license along with this
work.  If not, see <http://creativecommons.org/licenses/by-nc-sa/3.0/>.

This program is based off work covered by the following copyright(s):
SourceBans 1.4.11
Copyright © 2007-2014 SourceBans Team - Part of GameConnect
Licensed under CC-BY-NC-SA 3.0
Page: <http://www.sourcebans.net/> - <http://www.gameconnect.net/>
*************************************************************************/

if (!defined("IN_SB")) {
    echo "Vous ne devriez pas être ici. Suivez uniquement les liens!";
    die();
}
global $userbank, $theme;

new AdminTabs([], $userbank, $theme);

if (!isset($_GET['id'])) {
    echo '<div id="msg-red" >
	<i class="fas fa-times fa-2x"></i>
	<b>Erreur</b>
	<br />
	Aucun id admin spécifié. Veuillez suivre uniquement les liens
</div>';
    PageDie();
}
$_GET['id'] = (int) $_GET['id'];

if (!$userbank->GetProperty("user", $_GET['id'])) {
    Log::add("e", "Échec de l'obtention des données de l'admin", "Impossible de trouver les données admin avec cet id $_GET[id].");
    echo '<div id="msg-red" >
	<i class="fas fa-times fa-2x"></i>
	<b>Erreur</b>
	<br />
	Erreur lors de l\'obtention des données actuelles.
</div>';
    PageDie();
}

// Skip all checks if root
if (!$userbank->HasAccess(ADMIN_OWNER)) {
    if (!$userbank->HasAccess(ADMIN_EDIT_ADMINS) || ($userbank->HasAccess(ADMIN_OWNER, $_GET['id']) && $_GET['id'] != $userbank->GetAid())) {
        Log::add("w", "Tentative de Hack", $userbank->GetProperty("user")." a voulu éditer les détails de ".$userbank->GetProperty('user', $_GET['id']).", mais a échoué.");
        echo '<div id="msg-red" >
		<i class="fas fa-times fa-2x"></i>
		<b>Erreur</b>
		<br />
		Vous n\'êtes pas autorisé à modifier d\'autres profils.
	</div>';
        PageDie();
    }
}
$errorScript = "";

// Form submitted?
if (isset($_POST['adminname'])) {
    $a_name           = trim($_POST['adminname']);
    $a_steam          = \SteamID\SteamID::toSteam2(trim($_POST['steam']));
    $a_email          = trim($_POST['email']);
    $a_serverpass     = isset($_POST['a_useserverpass']) && $_POST['a_useserverpass'] == "on";
    $pw_changed       = false;
    $serverpw_changed = false;

    // Form validation
    $error = 0;

    // Check name
    if (empty($a_name)) {
        $error++;
        $errorScript .= "$('adminname.msg').innerHTML = 'Vous devez saisir un nom pour l'admin.';";
        $errorScript .= "$('adminname.msg').setStyle('display', 'block');";
    } else {
        if (strstr($a_name, "'")) {
            $error++;
            $errorScript .= "$('adminname.msg').innerHTML = 'Un nom d'admin ne peut pas contenir de \" \' \".';";
            $errorScript .= "$('adminname.msg').setStyle('display', 'block');";
        } else {
            if ($a_name != $userbank->GetProperty('user', $_GET['id']) && $userbank->isNameTaken($a_name)) {
                $error++;
                $errorScript .= "$('adminname.msg').innerHTML = 'Un admin avec ce nom existe déjà.';";
                $errorScript .= "$('adminname.msg').setStyle('display', 'block');";
            }
        }
    }

    // If they didnt type a steamid
    if ((empty($a_steam) || strlen($a_steam) < 10)) {
        $error++;
        $errorScript .= "$('steam.msg').innerHTML = 'Vous devez entrer un Steam ID ou ID communautaire pour l'admin.';";
        $errorScript .= "$('steam.msg').setStyle('display', 'block');";
    } else {
        // Validate the steamid or fetch it from the community id
        if (!\SteamID\SteamID::isValidID($a_steam)) {
            $error++;
            $errorScript .= "$('steam.msg').innerHTML = 'Veuillez entrer un Steam ID ou ID communautaire valide.';";
            $errorScript .= "$('steam.msg').setStyle('display', 'block');";
        } else {
            // Is an other admin already registred with that steam id?
            if ($a_steam != $userbank->GetProperty('authid', $_GET['id']) && $userbank->isSteamIDTaken($a_steam)) {
                $admins = $userbank->GetAllAdmins();
                foreach ($admins as $admin) {
                    if ($admin['authid'] == $a_steam) {
                        $name = $admin['user'];
                        break;
                    }
                }
                $error++;
                $errorScript .= "$('steam.msg').innerHTML = 'L'admin " . htmlspecialchars(addslashes($name)) . " utilise déjà ce Steam ID.';";
                $errorScript .= "$('steam.msg').setStyle('display', 'block');";
            }
        }
    }

    // No email
    if (empty($a_email)) {
        // Only required, if admin has web permissions.
        if ($GLOBALS['userbank']->GetProperty('extraflags', $_GET['id']) != 0 || $GLOBALS['userbank']->GetProperty('gid', $_GET['id']) > 0) {
            $error++;
            $errorScript .= "$('email.msg').innerHTML = 'Vous devez saisir une adresse e-mail.';";
            $errorScript .= "$('email.msg').setStyle('display', 'block');";
        }
    } else {
        // Is an other admin already registred with that email address?
        if ($a_email != $userbank->GetProperty('email', $_GET['id']) && $userbank->isEmailTaken($a_email)) {
            $admins = $userbank->GetAllAdmins();
            foreach ($admins as $admin) {
                if ($admin['email'] == $a_email) {
                    $name = $admin['user'];
                    break;
                }
            }
            $error++;
            $errorScript .= "$('email.msg').innerHTML = 'Cette adresse e-mail est déjà utilisée par " . htmlspecialchars(addslashes($name)) . ".';";
            $errorScript .= "$('email.msg').setStyle('display', 'block');";
        }
    }

    // Only validate passwords, if admin has access to edit it at all
    if ($userbank->HasAccess(ADMIN_OWNER) || $_GET['id'] == $userbank->GetAid()) {
        // Don't change the password, if not set
        if (!empty($_POST['password'])) {
            $pw_changed = true;
            // DID type a password, so he wants to change it.
            // Password too short?
            if (strlen($_POST['password']) < MIN_PASS_LENGTH) {
                $error++;
                $errorScript .= "$('password.msg').innerHTML = 'Votre mot de passe doit contenir au minimum " . MIN_PASS_LENGTH . " caractères.';";
                $errorScript .= "$('password.msg').setStyle('display', 'block');";
            } else {
                // No confirmation typed
                if (empty($_POST['password2'])) {
                    $error++;
                    $errorScript .= "$('password2.msg').innerHTML = 'Vous devez confirmer le mot de passe.';";
                    $errorScript .= "$('password2.msg').setStyle('display', 'block');";
                } elseif ($_POST['password'] != $_POST['password2']) {
                    // Passwords match?
                    $error++;
                    $errorScript .= "$('password2.msg').innerHTML = 'Vos mots de passe ne correspondent pas.';";
                    $errorScript .= "$('password2.msg').setStyle('display', 'block');";
                }
            }
        }

        // Check for the serverpassword
        if (isset($_POST['a_useserverpass']) && $_POST['a_useserverpass'] == "on") {
            if (!empty($_POST['a_serverpass'])) {
                $serverpw_changed = true;
            }

            // No password given and no set before?
            $srvpw = $userbank->GetProperty('srv_password', $_GET['id']);
            if (empty($_POST['a_serverpass']) && empty($srvpw)) {
                $error++;
                $errorScript .= "$('a_serverpass.msg').innerHTML = 'Vous devez saisir un mot de passe serveur ou décocher la case.';";
                $errorScript .= "$('a_serverpass.msg').setStyle('display', 'block');";
            } elseif (strlen($_POST['a_serverpass']) < MIN_PASS_LENGTH) {
                // Password too short?
                $error++;
                $errorScript .= "$('a_serverpass.msg').innerHTML = 'Votre mot de passe doit contenir au minimum " . MIN_PASS_LENGTH . " caractères.';";
                $errorScript .= "$('a_serverpass.msg').setStyle('display', 'block');";
            }
        }
    }

    // Only proceed, if there are no errors in the form
    if ($error == 0) {
        // set the basic fields
        $edit = $GLOBALS['db']->Execute(
            "UPDATE " . DB_PREFIX . "_admins SET
            `user` = ?, `authid` = ?, `email` = ?
            WHERE `aid` = ?",
            array(
                $a_name,
                $a_steam,
                $a_email,
                $_GET['id']
            )
        );

        // Password changed?
        if ($pw_changed) {
            $edit = $GLOBALS['db']->Execute(
                "UPDATE " . DB_PREFIX . "_admins SET
                `password` = ?
                WHERE `aid` = ?",
                array(
                    password_hash($_POST['password'], PASSWORD_BCRYPT),
                    $_GET['id']
                )
            );
        }

        // Server Admin Password changed?
        if ($serverpw_changed) {
            $edit = $GLOBALS['db']->Execute(
                "UPDATE " . DB_PREFIX . "_admins SET
                `srv_password` = ?
                WHERE `aid` = ?",
                array(
                    $_POST['a_serverpass'],
                    $_GET['id']
                )
            );
        } elseif (isset($_POST['a_useserverpass']) && $_POST['a_useserverpass'] != "on") {
            // Remove the server password
            $edit = $GLOBALS['db']->Execute(
                "UPDATE " . DB_PREFIX . "_admins SET
                `srv_password` = NULL
                WHERE `aid` = ?",
                array(
                    $_GET['id']
                )
            );
        }

        // to prevent rehash window to error with "no access", cause pw doesn't match
        $ownpwchanged = false;
        if ($_GET['id'] == $userbank->GetAid() && !empty($_POST['password']) && password_verify($_POST['password'], $userbank->GetProperty("password"))) {
            $ownpwchanged = true;
        }

        if (Config::getBool('config.enableadminrehashing')) {
            // rehash the admins on the servers
            $serveraccessq = $GLOBALS['db']->GetAll("SELECT s.sid FROM `" . DB_PREFIX . "_servers` s
                LEFT JOIN `" . DB_PREFIX . "_admins_servers_groups` asg ON asg.admin_id = '" . (int) $_GET['id'] . "'
                LEFT JOIN `" . DB_PREFIX . "_servers_groups` sg ON sg.group_id = asg.srv_group_id
                WHERE ((asg.server_id != '-1' AND asg.srv_group_id = '-1')
                OR (asg.srv_group_id != '-1' AND asg.server_id = '-1'))
                AND (s.sid IN(asg.server_id) OR s.sid IN(sg.server_id)) AND s.enabled = 1");
            $allservers    = [];
            foreach ($serveraccessq as $access) {
                if (!in_array($access['sid'], $allservers)) {
                    $allservers[] = $access['sid'];
                }
            }
            $rehashing = true;
        }

        $admname = $GLOBALS['db']->GetRow("SELECT user FROM `" . DB_PREFIX . "_admins` WHERE aid = ?", array(
            (int) $_GET['id']
        ));
        Log::add("m", "Détails de l'admin mis à jour", "Les détails de ($admname[user]) ont été mis à jour.");
        if ($ownpwchanged) {
            echo '<script>ShowBox("Détails de l\'admin mis à jour", "Les détails de l\admin ont bien été mis à jour", "green", "index.php?p=login");TabToReload();</script>';
        } elseif (isset($rehashing)) {
            echo '<script>ShowRehashBox("' . implode(",", $allservers) . '", "Détails de l\'admin mis à jour", "Les détails de l\'admin ont été mis à jour avec succès", "green", "index.php?p=admin&c=admins");TabToReload();</script>';
        } else {
            echo '<script>ShowBox("Détails de l\'admin mis à jour", "Les détails de l\'admin ont été mis à jour avec succès", "green", "index.php?p=admin&c=admins");TabToReload();</script>';
        }
    }
} else {
    // get current values
    $a_name = $userbank->GetProperty("user", $_GET['id']);
    $a_steam = trim($userbank->GetProperty("authid", $_GET['id']));
    $a_email = $userbank->GetProperty("email", $_GET['id']);
    $a_serverpass = $userbank->GetProperty("srv_password", $_GET['id']);
    $a_serverpass = !empty($a_serverpass);
}

$theme->assign('change_pass', ($userbank->HasAccess(ADMIN_OWNER) || $userbank->HasAccess(ADMIN_EDIT_ADMINS)|| $_GET['id'] == $userbank->GetAid()));
$theme->assign('user', $a_name);
$theme->assign('authid', $a_steam);
$theme->assign('email', $a_email);
$theme->assign('a_spass', $a_serverpass);

$theme->display('page_admin_edit_admins_details.tpl');
?>
<script type="text/javascript">window.addEvent('domready', function(){
<?=$errorScript?>
});
</script>
