<?php
global $userbank, $theme;

$navbar = [
     [
        'title' => 'Tableau de bord',
        'endpoint' => 'home',
        'description' => 'Cette page affiche un aperçu des joueurs bannis, mutés et de vos serveurs.',
        'permission' => true
    ],
    [
        'title' => 'Serveurs',
        'endpoint' => 'servers',
        'description' => 'Tous vos serveurs et leur statut peuvent être consultés ici.',
        'permission' => true
    ],
    [
        'title' => 'Bans',
        'endpoint' => 'banlist',
        'description' => 'Tous les bans de la base de données peuvent être consultés à partir d\'ici.',
        'permission' => true
    ],
    [
        'title' => 'Mutes',
        'endpoint' => 'commslist',
        'description' => 'Tous les mutes écris et vocaux peuvent être consultés à partir d\'ici.',
        'permission' => Config::getBool('config.enablecomms')
    ],
    [
        'title' => 'Soumettre',
        'endpoint' => 'submit',
        'description' => 'Vous pouvez soumettre une demande de sanction envers un joueur ici. Votre demande sera ensuite étudié par les admins.',
        'permission' => Config::getBool('config.enablesubmit')
    ],
    [
        'title' => 'Protester',
        'endpoint' => 'protest',
        'description' => 'Vous pouvez protester votre ban ici et pourquoi vous devriez être débanni.',
        'permission' => Config::getBool('config.enableprotest')
    ],
    [
        'title' => 'Panel admin',
        'endpoint' => 'admin',
        'description' => 'Ceci est le panneau de contrôle de SourceBans où vous pouvez configurer de nouveaux admins, ajouter un nouveau serveur, etc.',
        'permission' => $userbank->is_admin()
    ]
];

$admin = [
    [
        'title' => 'Admins',
        'endpoint' => 'admins',
        'permission' => ADMIN_OWNER|ADMIN_LIST_ADMINS|ADMIN_ADD_ADMINS|ADMIN_EDIT_ADMINS|ADMIN_DELETE_ADMINS
    ],
    [
        'title' => 'Serveurs',
        'endpoint' => 'servers',
        'permission' => ADMIN_OWNER|ADMIN_LIST_SERVERS|ADMIN_ADD_SERVER|ADMIN_EDIT_SERVERS|ADMIN_DELETE_SERVERS
    ],
    [
        'title' => 'Bans',
        'endpoint' => 'bans',
        'permission' => ADMIN_OWNER|ADMIN_ADD_BAN|ADMIN_EDIT_OWN_BANS|ADMIN_EDIT_GROUP_BANS|ADMIN_EDIT_ALL_BANS|ADMIN_BAN_PROTESTS|ADMIN_BAN_SUBMISSIONS
    ],
    [
        'title' => 'Mutes',
        'endpoint' => 'comms',
        'permission' => ADMIN_OWNER|ADMIN_ADD_BAN|ADMIN_EDIT_OWN_BANS|ADMIN_EDIT_ALL_BANS
    ],
    [
        'title' => 'Groupes',
        'endpoint' => 'groups',
        'permission' => ADMIN_OWNER|ADMIN_LIST_GROUPS|ADMIN_ADD_GROUP|ADMIN_EDIT_GROUPS|ADMIN_DELETE_GROUPS
    ],
    [
        'title' => 'Paramètres',
        'endpoint' => 'settings',
        'permission' => ADMIN_OWNER|ADMIN_WEB_SETTINGS
    ],
    [
        'title' => 'Mods',
        'endpoint' => 'mods',
        'permission' => ADMIN_OWNER|ADMIN_LIST_MODS|ADMIN_ADD_MODS|ADMIN_EDIT_MODS|ADMIN_DELETE_MODS
    ]
];

$active = filter_input(INPUT_GET, 'p', FILTER_SANITIZE_SPECIAL_CHARS);
foreach ($navbar as $key => $tab) {
    $navbar[$key]['state'] = ($active === $tab['endpoint']) ? 'active' : 'nonactive';

    if (!$tab['permission']) {
        unset($navbar[$key]);
    }
}

if ($userbank->is_admin()) {
    $cat = filter_input(INPUT_GET, 'c', FILTER_SANITIZE_SPECIAL_CHARS);
    foreach ($admin as $key => $tab) {
        $admin[$key]['state'] = ($cat === $tab['endpoint']) ? 'active' : '';

        if (!$userbank->HasAccess($tab['permission'])) {
            unset($admin[$key]);
        }
    }
}

$theme->assign('navbar', array_values($navbar));
$theme->assign('adminbar', array_values($admin));
$theme->assign('isAdmin', $userbank->is_admin());
$theme->assign('login', $userbank->is_logged_in());
$theme->assign('username', $userbank->GetProperty("user"));
$theme->display('core/navbar.tpl');
