<?php
/*************************************************************************
This file is part of SourceBans++

SourceBans++ (c) 2014-2023 by SourceBans++ Dev Team

The SourceBans++ Web panel is licensed under a
Creative Commons Attribution-NonCommercial-ShareAlike 3.0 Unported License.

You should have received a copy of the license along with this
work.  If not, see <http://creativecommons.org/licenses/by-nc-sa/3.0/>.

This program is based off work covered by the following copyright(s):
SourceBans 1.4.11
Copyright © 2007-2014 SourceBans Team - Part of GameConnect
Licensed under CC-BY-NC-SA 3.0
Page: <http://www.sourcebans.net/> - <http://www.gameconnect.net/>
 *************************************************************************/
/**
 * SourceBans "Error Connecting()" Debug
 * Checks for the ports being forwarded correctly
 */

/**
 * Config part
 * Change to IP and port of the gameserver you want to test
 */
$serverip   = "";
$serverport = 27015;
// You only need to specify this if you want to test the rcon tcp connection
// Leave blank if it's only the serverinfo erroring.
$serverrcon = "";


/*******
 * Don't change below here 
*******/

if (empty($serverip) || empty($serverport)) {
    die('[-] Aucune information sur le serveur définie. Ouvrez ce fichier et spécifiez l\'IP et port de votre serveur de jeu.');
}

echo '[+] SourceBans "Erreur de connexion()" Debug démarré pour le serveur' . $serverip . ':' . $serverport . '<br /><br />';

// Check for UDP connection being available and writable
echo '[+] Tente d\'établir une connexion UDP<br />';
$sock = @fsockopen("udp://" . $serverip, $serverport, $errno, $errstr, 2);

$isBanned = false;

if (!$sock) {
    echo '[-] Erreur de connexion #' . $errno . ': ' . $errstr . '<br />';
} else {
    echo '[+] Connexion UDP réussie!<br />';

    stream_set_timeout($sock, 1);
    // Try to get serverinformation
    echo '[+] Tente d\'écrire sur le socket<br />';
    $written = fwrite($sock, "\xFF\xFF\xFF\xFF\x54Source Engine Query\0");
    if ($written === false) {
        echo '[-] Erreur d\'écriture.<br />';
    } else {
        echo '[+] Informations d\'écriture serveur reçues avec succès. (Cela ne veut rien dire sur un flux UDP.)<br />';
        $packet = fread($sock, 1480);

        if (empty($packet)) {
            echo '[-] Erreur lors de l\'obtention des informations sur le serveur. Impossible de lire à partir du flux UDP. Le port est peut-être bloqué.<br />';
        } else {
            if (substr($packet, 5, (strpos(substr($packet, 5), "\0") - 1)) == "Banned by server") {
                echo '[-] Vous avez reçu une réponse, mais l\'adresse IP de ce serveur Web est interdite par le serveur.<br />';
                $isBanned = true;
            } else {
                $packet   = substr($packet, 6);
                $hostname = substr($packet, 0, strpos($packet, "\0"));
                echo '[+] J\'ai une réponse ! Serveur: ' . $hostname . ' <br />';
            }
        }
    }
    fclose($sock);
}

echo '<br />';

// Check for TCP connection being available and writeable
echo '[+] Tente d\'établir une connexion TCP<br />';
$sock = @fsockopen($serverip, $serverport, $errno, $errstr, 2);
if (!$sock) {
    echo '[-] Erreur de connexion #' . $errno . ': ' . $errstr . '<br />';
} else {
    echo '[+] Connexion TCP réussie!<br />';
    if (empty($serverrcon)) {
        echo '[o] Stoppé ici car aucun mot de passe rcon n\'est spécifié.';
    } else if ($isBanned) {
        echo '[o] Stoppé ici car cette adresse IP est interdite par le serveur de jeu.';
    } else {
        stream_set_timeout($sock, 2);
        $data = pack("VV", 0, 03) . $serverrcon . chr(0) . '' . chr(0);
        $data = pack("V", strlen($data)) . $data;

        echo '[+] Tente d\'écrire sur le socket TCP et de s\'authentifier via rcon<br />';
        $written = fwrite($sock, $data, strlen($data));

        if ($written === false) {
            echo '[-] Erreur d\'écriture.<br />';
        } else {
            echo '[+] Demande d\'authentification envoyée avec succès.<br />';
            $size = fread($sock, 4);
            if (!$size) {
                echo '[-] Erreur d\'écriture.<br />';
            } else {
                echo '[+] J\'ai une réponse! <br />';
                $size   = unpack('V1Size', $size);
                $packet = fread($sock, $size["Size"]);
                $size   = fread($sock, 4);
                $size   = unpack('V1Size', $size);
                $packet = fread($sock, $size["Size"]);
                $ret    = unpack("V1ID/V1Reponse/a*S1/a*S2", $packet);
                if (empty($ret) || (isset($ret['ID']) && $ret['ID'] == -1)) {
                    echo '[-] Mauvais mot de passe ;) N\'essayez pas cela trop souvent ou votre serveur Web sera banni par le serveur de jeu.<br />';
                } else {
                    echo '[+] Mot de passe correct!';
                }
            }
        }
    }
    fclose($sock);
}
